@@ -21,7 +21,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
 #include <stdio.h>
 #include <libgda/gda-client.h>
 #include <libgda/gda-config.h>
@@ -35,6 +34,7 @@
 struct _GdaConnectionPrivate {
 	GdaClient *client;
 	GdaServerProvider *provider_obj;
+	GdaConnectionOptions options;
 	gchar *dsn;
 	gchar *cnc_string;
 	gchar *provider;
@@ -61,20 +61,24 @@ static GObjectClass *parent_class = NULL;
  * Callbacks
  */
 
-//static void
-//recset_weak_cb (gpointer user_data, GObject *object)
-//{
-//	GdaRecordset *recset = (GdaRecordset *) object;
-//	GdaConnection *cnc = (GdaConnection *) user_data;
-//
-//	g_return_if_fail (GDA_IS_RECORDSET (recset));
-//	g_return_if_fail (GDA_IS_CONNECTION (cnc));
-//
-//	cnc->priv->recset_list = g_list_remove (cnc->priv->recset_list, recset);
-//}
+#if 0
+/* CODE IS DISABLED */
+static void
+recset_weak_cb (gpointer user_data, GObject *object)
+{
+	GdaRecordset *recset = (GdaRecordset *) object;
+	GdaConnection *cnc = (GdaConnection *) user_data;
+
+	g_return_if_fail (GDA_IS_RECORDSET (recset));
+	g_return_if_fail (GDA_IS_CONNECTION (cnc));
+
+	cnc->priv->recset_list = g_list_remove (cnc->priv->recset_list, recset);
+}
+#endif
 
 /*
  * GdaConnection class implementation
+ * @klass:
  */
 
 static void
@@ -125,6 +129,7 @@ gda_connection_finalize (GObject *object)
 	if (cnc->priv->is_open) {
 		/* close the connection to the provider */
 		gda_server_provider_close_connection (cnc->priv->provider_obj, cnc);
+		gda_client_notify_connection_closed_event (cnc->priv->client, cnc);
 	}
 
 	g_object_unref (G_OBJECT (cnc->priv->provider_obj));
@@ -136,8 +141,7 @@ gda_connection_finalize (GObject *object)
 	g_free (cnc->priv->username);
 	g_free (cnc->priv->password);
 
-	g_list_foreach (cnc->priv->error_list, (GFunc) gda_error_free, NULL);
-	g_list_free (cnc->priv->error_list);
+	gda_error_list_free (cnc->priv->error_list);
 
 	g_list_foreach (cnc->priv->recset_list, (GFunc) g_object_unref, NULL);
 
@@ -148,27 +152,31 @@ gda_connection_finalize (GObject *object)
 	parent_class->finalize (object);
 }
 
+/**
+ * gda_connection_get_type
+ * 
+ * Register the GdaConnection class on the GLib type system.
+ * 
+ * Returns: the GType identifying the class.
+ */
 GType
 gda_connection_get_type (void)
 {
-	static GType type = 0;
+   static GType type = 0;
 
-	if (!type) {
-		if (type == 0) {
-			static GTypeInfo info = {
-				sizeof (GdaConnectionClass),
-				(GBaseInitFunc) NULL,
-				(GBaseFinalizeFunc) NULL,
-				(GClassInitFunc) gda_connection_class_init,
-				NULL, NULL,
-				sizeof (GdaConnection),
-				0,
-				(GInstanceInitFunc) gda_connection_init
-			};
-			type = g_type_register_static (PARENT_TYPE, "GdaConnection", &info, 0);
-		}
+	if (type == 0) {
+		static GTypeInfo info = {
+			sizeof (GdaConnectionClass),
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) gda_connection_class_init,
+			NULL, NULL,
+			sizeof (GdaConnection),
+			0,
+			(GInstanceInitFunc) gda_connection_init
+		};
+	type = g_type_register_static (PARENT_TYPE, "GdaConnection", &info, 0);
 	}
-
 	return type;
 }
 
@@ -179,6 +187,7 @@ gda_connection_get_type (void)
  * @dsn: GDA data source to connect to.
  * @username: user name to use to connect.
  * @password: password for @username.
+ * @options: options for the connection.
  *
  * This function creates a new #GdaConnection object. It is not
  * intended to be used directly by applications (use
@@ -191,11 +200,14 @@ gda_connection_new (GdaClient *client,
 		    GdaServerProvider *provider,
 		    const gchar *dsn,
 		    const gchar *username,
-		    const gchar *password)
+		    const gchar *password,
+		    GdaConnectionOptions options)
 {
 	GdaConnection *cnc;
 	GdaDataSourceInfo *dsn_info;
 	GdaQuarkList *params;
+	const char *real_username = NULL;
+	const char *real_password = NULL;
 
 	g_return_val_if_fail (GDA_IS_CLIENT (client), NULL);
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
@@ -207,6 +219,34 @@ gda_connection_new (GdaClient *client,
 		return NULL;
 	}
 
+	params = gda_quark_list_new_from_string (dsn_info->cnc_string);
+
+	/* retrieve correct username/password */
+	if (username)
+		real_username = username;
+	else {
+		if (dsn_info->username)
+			real_username = dsn_info->username;
+		else {
+			real_username = gda_quark_list_find (params, "USER");
+			if (real_username)
+				gda_quark_list_remove (params, "USER");
+		}
+	}
+
+	if (password)
+		real_password = password;
+	else {
+		if (dsn_info->password)
+			real_password = dsn_info->password;
+		else {
+			real_password = gda_quark_list_find (params, "PASSWORD");
+			if (real_password)
+				gda_quark_list_remove (params, "PASSWORD");
+		}
+	}
+
+	/* create the connection object */
 	cnc = g_object_new (GDA_TYPE_CONNECTION, NULL);
 
 	gda_connection_set_client (cnc, client);
@@ -215,19 +255,37 @@ gda_connection_new (GdaClient *client,
 	cnc->priv->dsn = g_strdup (dsn);
 	cnc->priv->cnc_string = g_strdup (dsn_info->cnc_string);
 	cnc->priv->provider = g_strdup (dsn_info->provider);
-	cnc->priv->username = g_strdup (username);
-	cnc->priv->password = g_strdup (password);
+	cnc->priv->username = g_strdup (real_username);
+	cnc->priv->password = g_strdup (real_password);
+	cnc->priv->options = options;
 
 	gda_config_free_data_source_info (dsn_info);
 
 	/* try to open the connection */
-	params = gda_quark_list_new_from_string (cnc->priv->cnc_string);
-	if (!gda_server_provider_open_connection (provider, cnc, params, username, password)) {
+	if (!gda_server_provider_open_connection (provider, cnc, params,
+						  cnc->priv->username,
+						  cnc->priv->password)) {
+		GList *errors_copy;
+
+		errors_copy = gda_connection_get_errors (cnc);
+		/* notify the GdaClient of the error, since
+		   that's the only way we can notify it of errors
+		   when creating the connection */
+		if (errors_copy) {
+			GList *l;
+
+			for (l = errors_copy; l != NULL; l = l->next)
+				gda_client_notify_error_event (client, cnc, GDA_ERROR (l->data));
+		}
 		gda_quark_list_free (params);
 		g_object_unref (G_OBJECT (cnc));
 		return NULL;
 	}
 
+	/* notify action */
+	gda_client_notify_connection_opened_event (client, cnc);
+
+	/* free memory */
 	gda_quark_list_free (params);
 	cnc->priv->is_open = TRUE;
 
@@ -303,6 +361,52 @@ gda_connection_set_client (GdaConnection *cnc, GdaClient *client)
 	cnc->priv->client = client;
 }
 
+/**
+ * gda_connection_get_options
+ * @cnc: a #GdaConnection object.
+ *
+ * Get the #GdaConnectionOptions used to open this connection.
+ *
+ + Returns: the connection options.
+*/
+GdaConnectionOptions
+gda_connection_get_options (GdaConnection *cnc)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), -1);
+	return cnc->priv->options;
+}
+
+/**
+ * gda_connection_get_server_version
+ * @cnc: A #GdaConnection object.
+ *
+ * Get the version string of the underlying database server.
+ *
+ * Returns: the server version string.
+ */
+const gchar *
+gda_connection_get_server_version (GdaConnection *cnc)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	return gda_server_provider_get_server_version (cnc->priv->provider_obj, cnc);
+}
+
+/**
+ * gda_connection_get_database
+ * @cnc: A #GdaConnection object.
+ *
+ * Get the name of the currently active database in the given
+ * @GdaConnection.
+ *
+ * Returns: the name of the current database.
+ */
+const gchar *
+gda_connection_get_database (GdaConnection *cnc)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	return gda_server_provider_get_database (cnc->priv->provider_obj, cnc);
+}
+
 /**
  * gda_connection_get_dsn
  * @cnc: A #GdaConnection object
@@ -393,17 +497,24 @@ gda_connection_get_password (GdaConnection *cnc)
  * function, the connection object (and the associated #GdaClient object)
  * emits the "error" signal, to which clients can connect to be
  * informed of errors.
+ *
+ * @error is stored internally, so you don't need to unref it.
  */
 void
 gda_connection_add_error (GdaConnection *cnc, GdaError *error)
 {
+	GList *err_list;
+
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
 	g_return_if_fail (GDA_IS_ERROR (error));
 
-	cnc->priv->error_list = g_list_append (cnc->priv->error_list, error);
+	err_list = cnc->priv->error_list;
+	gda_error_list_free (err_list);
+	
+	err_list = g_list_append (NULL, error);
+	cnc->priv->error_list = err_list;
 
 	g_signal_emit (G_OBJECT (cnc), gda_connection_signals[ERROR], 0, cnc->priv->error_list);
-	cnc->priv->error_list = NULL;
 }
 
 /**
@@ -434,7 +545,7 @@ gda_connection_add_error_string (GdaConnection *cnc, const gchar *str, ...)
 	error = gda_error_new ();
 	gda_error_set_description (error, sz);
 	gda_error_set_number (error, -1);
-	gda_error_set_source (error, g_get_prgname ());
+	gda_error_set_source (error, gda_connection_get_provider (cnc));
 	gda_error_set_sqlstate (error, "-1");
 	
 	gda_connection_add_error (cnc, error);
@@ -452,6 +563,8 @@ gda_connection_add_error_string (GdaConnection *cnc, const gchar *str, ...)
  * signal. The only difference is that, instead of a notification
  * for each error, this function only does one notification for
  * the whole list of errors.
+ *
+ * @error_list is copied to an internal list and freed.
  */
 void
 gda_connection_add_error_list (GdaConnection *cnc, GList *error_list)
@@ -461,16 +574,34 @@ gda_connection_add_error_list (GdaConnection *cnc, GList *error_list)
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
 	g_return_if_fail (error_list != NULL);
 
-	for (l = error_list; l; l = l->next) {
-		GdaError *error = GDA_ERROR (l->data);
-		cnc->priv->error_list = g_list_append (cnc->priv->error_list, error);
-	}
+	l = cnc->priv->error_list;
+	gda_error_list_free (l);
+	l = gda_error_list_copy (error_list);
 
+	cnc->priv->error_list = l;
 	/* notify errors */
-	g_signal_emit (G_OBJECT (cnc), gda_connection_signals[ERROR], 0, cnc->priv->error_list);
-	cnc->priv->error_list = NULL;
+	g_signal_emit (G_OBJECT (cnc), gda_connection_signals[ERROR], 0, l);
 
-	g_list_free (error_list);
+	gda_error_list_free (error_list);
+}
+
+/**
+ * gda_connection_change_database
+ * @cnc: a #GdaConnection object.
+ * @name: name of database to switch to.
+ *
+ * Change the current database for the given connection. This operation
+ * is not available in all providers.
+ *
+ * Returns: TRUE if successful, FALSE otherwise.
+ */
+gboolean
+gda_connection_change_database (GdaConnection *cnc, const gchar *name)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (name != NULL, FALSE);
+
+	return gda_server_provider_change_database (cnc->priv->provider_obj, cnc, name);
 }
 
 /**
@@ -555,7 +686,7 @@ gda_connection_execute_command (GdaConnection *cnc,
  * a list of them, as is the case with #gda_connection_execute_command.
  *
  * Returns: a #GdaDataModel containing the data returned by the
- * data source.
+ * data source, or %NULL on error.
  */
 GdaDataModel *
 gda_connection_execute_single_command (GdaConnection *cnc,
@@ -581,15 +712,51 @@ gda_connection_execute_single_command (GdaConnection *cnc,
 	return model;
 }
 
+/**
+ * gda_connection_execute_non_query
+ * @cnc: a #GdaConnection object.
+ * @cmd: a #GdaCommand.
+ * @params: parameter list.
+ *
+ * Execute a single command on the underlying database, and get the
+ * number of rows affected.
+ *
+ * Returns: the number of affected rows by the executed command,
+ * or -1 on error.
+ */
+gint
+gda_connection_execute_non_query (GdaConnection *cnc,
+				  GdaCommand *cmd,
+				  GdaParameterList *params)
+{
+	GList *reclist;
+	GdaDataModel *model;
+	gint result = -1;
+
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), -1);
+	g_return_val_if_fail (cmd != NULL, -1);
+
+	reclist = gda_connection_execute_command (cnc, cmd, params);
+	if (!reclist)
+		return -1;
+
+	model = (GdaDataModel *) reclist->data;
+	if (GDA_IS_DATA_MODEL (model))
+		result = gda_data_model_get_n_rows (model);
+
+	g_list_foreach (reclist, (GFunc) g_object_unref, NULL);
+	g_list_free (reclist);
+
+	return result;
+}
+
 /**
  * gda_connection_begin_transaction
  * @cnc: a #GdaConnection object.
- * @id: transaction id.
+ * @xaction: a #GdaTransaction object.
  *
  * Start a transaction on the data source, identified by the
- * @id parameter, which, if not NULL, will be used by the underlying
- * provider to start a named transaction, if the data source it
- * is connected to does support that concept.
+ * @xaction parameter.
  *
  * Before starting a transaction, you can check whether the underlying
  * provider does support transactions or not by using the
@@ -599,37 +766,50 @@ gda_connection_execute_single_command (GdaConnection *cnc,
  * otherwise.
  */
 gboolean
-gda_connection_begin_transaction (GdaConnection *cnc, const gchar *id)
+gda_connection_begin_transaction (GdaConnection *cnc, GdaTransaction *xaction)
 {
+	gboolean retval;
+
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	return gda_server_provider_begin_transaction (cnc->priv->provider_obj, cnc, id);
+
+	retval = gda_server_provider_begin_transaction (cnc->priv->provider_obj, cnc, xaction);
+	if (retval)
+		gda_client_notify_transaction_started_event (cnc->priv->client, cnc, xaction);
+
+	return retval;
 }
 
 /**
  * gda_connection_commit_transaction
  * @cnc: a #GdaConnection object.
- * @id: transaction id.
+ * @xaction: a #GdaTransaction object.
  *
- * Commit the current transaction (if @id is NULL) or the named
- * transaction specified by the @id parameter.
+ * Commit the given transaction to the backend database.  You need to do 
+ * gda_connection_begin_transaction() first.
  *
  * Returns: TRUE if the transaction was finished successfully,
  * FALSE otherwise.
  */
 gboolean
-gda_connection_commit_transaction (GdaConnection *cnc, const gchar *id)
+gda_connection_commit_transaction (GdaConnection *cnc, GdaTransaction *xaction)
 {
+	gboolean retval;
+
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	return gda_server_provider_commit_transaction (cnc->priv->provider_obj, cnc, id);
+
+	retval = gda_server_provider_commit_transaction (cnc->priv->provider_obj, cnc, xaction);
+	if (retval)
+		gda_client_notify_transaction_committed_event (cnc->priv->client, cnc, xaction);
+
+	return retval;
 }
 
 /**
  * gda_connection_rollback_transaction
  * @cnc: a #GdaConnection object.
- * @id: transaction id.
+ * @xaction: a #GdaTransaction object.
  *
- * Rollback the current transaction (if @id is NULL) or the named
- * transaction specified by the @id parameter. This means that all changes
+ * Rollback the given transaction. This means that all changes
  * made to the underlying data source since the last call to
  * #gda_connection_begin_transaction or #gda_connection_commit_transaction
  * will be discarded.
@@ -637,10 +817,16 @@ gda_connection_commit_transaction (GdaConnection *cnc, const gchar *id)
  * Returns: TRUE if the operation was successful, FALSE otherwise.
  */
 gboolean
-gda_connection_rollback_transaction (GdaConnection *cnc, const gchar *id)
+gda_connection_rollback_transaction (GdaConnection *cnc, GdaTransaction *xaction)
 {
+	gboolean retval;
+
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	return gda_server_provider_rollback_transaction (cnc->priv->provider_obj, cnc, id);
+	retval = gda_server_provider_rollback_transaction (cnc->priv->provider_obj, cnc, xaction);
+	if (retval)
+		gda_client_notify_transaction_cancelled_event (cnc->priv->client, cnc, xaction);
+
+	return retval;
 }
 
 /**
@@ -648,7 +834,7 @@ gda_connection_rollback_transaction (GdaConnection *cnc, const gchar *id)
  * @cnc: a #GdaConnection object.
  * @feature: feature to ask for.
  *
- * Ask the underlying provider for a specific supported feature.
+ * Ask the underlying provider for if a specific feature is supported.
  *
  * Returns: TRUE if the provider supports it, FALSE if not.
  */
@@ -686,3 +872,21 @@ gda_connection_get_schema (GdaConnection *cnc,
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	return gda_server_provider_get_schema (cnc->priv->provider_obj, cnc, schema, params);
 }
+
+/**
+ * gda_connection_get_errors
+ * @cnc: a #GdaConnection.
+ *
+ * Retrieves a list of the last errors ocurred in the connection.
+ * You can make a copy of the list using #gda_error_list_copy.
+ * 
+ * Returns: a GList of #GdaError.
+ *
+ */
+const GList *
+gda_connection_get_errors (GdaConnection *cnc)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	return cnc->priv->error_list;
+}
+
