@@ -21,10 +21,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
 #include <libgda/gda-data-model-array.h>
 #include <libgda/gda-intl.h>
 #include <stdlib.h>
+#include <string.h>
 #include "gda-oracle.h"
 #include "gda-oracle-provider.h"
 #include "gda-oracle-recordset.h"
@@ -38,6 +38,7 @@ static void gda_oracle_provider_init       (GdaOracleProvider *provider,
 					   GdaOracleProviderClass *klass);
 static void gda_oracle_provider_finalize   (GObject *object);
 
+static const gchar *gda_oracle_provider_get_version (GdaServerProvider *provider);
 static gboolean gda_oracle_provider_open_connection (GdaServerProvider *provider,
 						    GdaConnection *cnc,
 						    GdaQuarkList *params,
@@ -45,6 +46,8 @@ static gboolean gda_oracle_provider_open_connection (GdaServerProvider *provider
 						    const gchar *password);
 static gboolean gda_oracle_provider_close_connection (GdaServerProvider *provider,
 						     GdaConnection *cnc);
+static const gchar *gda_oracle_provider_get_server_version (GdaServerProvider *provider,
+							    GdaConnection *cnc);
 static const gchar *gda_oracle_provider_get_database (GdaServerProvider *provider,
 						     GdaConnection *cnc);
 static gboolean gda_oracle_provider_create_database (GdaServerProvider *provider,
@@ -100,8 +103,10 @@ gda_oracle_provider_class_init (GdaOracleProviderClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_oracle_provider_finalize;
+	provider_class->get_version = gda_oracle_provider_get_version;
 	provider_class->open_connection = gda_oracle_provider_open_connection;
 	provider_class->close_connection = gda_oracle_provider_close_connection;
+	provider_class->get_server_version = gda_oracle_provider_get_server_version;
 	provider_class->get_database = gda_oracle_provider_get_database;
 	provider_class->create_database = gda_oracle_provider_create_database;
 	provider_class->drop_database = gda_oracle_provider_drop_database;
@@ -160,6 +165,13 @@ gda_oracle_provider_new (void)
 	return GDA_SERVER_PROVIDER (provider);
 }
 
+/* get_version handler for the GdaOracleProvider class */
+static const gchar *
+gda_oracle_provider_get_version (GdaServerProvider *provider)
+{
+	return VERSION;
+}
+
 /* open_connection handler for the GdaOracleProvider class */
 static gboolean
 gda_oracle_provider_open_connection (GdaServerProvider *provider,
@@ -259,7 +271,7 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 	if (strlen(username) > 0)
 		ora_username = username;
 	else
-		ora_username = gda_quark_list_find (params, "USERNAME");
+		ora_username = gda_quark_list_find (params, "USER");
 
 	/* if the username isn't provided, try to find it in the DSN */
 	if (strlen(password) > 0)
@@ -368,6 +380,10 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 		return FALSE;
 	}
 
+	priv_data->schema = g_ascii_strup(ora_username, -1);
+	priv_data->tables = NULL;
+	priv_data->views = NULL;
+	
 	/* attach the oracle connection data to the gda connection object */
 	g_object_set_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE, priv_data);
 
@@ -411,6 +427,12 @@ gda_oracle_provider_close_connection (GdaServerProvider *provider, GdaConnection
 		OCIHandleFree ((dvoid *) priv_data->herr, OCI_HTYPE_ERROR);
 	if (priv_data->henv)
 		OCIHandleFree ((dvoid *) priv_data->henv, OCI_HTYPE_ENV);
+	if (priv_data->schema)
+	    g_free(priv_data->schema);
+	if (priv_data->tables)
+	    g_tree_destroy(priv_data->tables);
+	if (priv_data->views)
+	    g_tree_destroy(priv_data->views);
 							
 	g_free (priv_data);
 	g_object_set_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE, NULL);
@@ -418,6 +440,26 @@ gda_oracle_provider_close_connection (GdaServerProvider *provider, GdaConnection
 	return TRUE;
 }
 
+/* get_server_version handler for the GdaOracleProvider class */
+static const gchar *
+gda_oracle_provider_get_server_version (GdaServerProvider *provider,
+					GdaConnection *cnc)
+{
+	GdaOracleConnectionData *priv_data;
+	static gchar version[512];
+
+	/* Get the OracleConnectionData */
+	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
+	if (!priv_data) {
+		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		return NULL;
+	}
+
+	OCIServerVersion (priv_data->hservice, priv_data->herr, version, 255, OCI_HTYPE_SVCCTX);
+
+	return (const gchar *) version;
+}
+
 /* 
  * prepare_oracle_statement prepares the Oracle statement handle for use 
  *
@@ -512,6 +554,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc,
 	GdaOracleConnectionData *priv_data;
 	gchar **arr;
 	gint result;
+	ub4 prefetch = 200;
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	if (!priv_data) {
@@ -544,7 +587,18 @@ process_sql_commands (GList *reclist, GdaConnection *cnc,
 				OCIHandleFree ((dvoid *) stmthp, OCI_HTYPE_STMT);
 				return NULL;
 			}
-				
+
+			result = OCIAttrSet (stmthp,
+					     OCI_HTYPE_STMT,
+					     &prefetch,
+					     0,
+					     OCI_ATTR_PREFETCH_ROWS,
+					     priv_data->herr);
+			if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
+					_("Could not set the Oracle statement pre-fetch row count"))) {
+				OCIHandleFree ((dvoid *) stmthp, OCI_HTYPE_STMT);
+				return NULL;
+			}
 
 			result = OCIStmtExecute (priv_data->hservice,
 						stmthp,
@@ -782,13 +836,18 @@ gda_oracle_provider_begin_transaction (GdaServerProvider *provider,
 	g_return_val_if_fail (xaction != NULL, FALSE);
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
-	ora_xaction = g_new0 (GdaOracleTransaction, 1);
-
 	if (!priv_data) {
 		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
+	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
+		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		return FALSE;
+	}
+
+	ora_xaction = g_new0 (GdaOracleTransaction, 1);
+
 	/* allocate the Oracle transaction handle */
 	result = OCIHandleAlloc ((dvoid *) priv_data->henv,
 				(dvoid **) &(ora_xaction->txnhp),
@@ -859,11 +918,16 @@ gda_oracle_provider_commit_transaction (GdaServerProvider *provider,
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	ora_xaction = g_object_get_data (G_OBJECT (xaction), OBJECT_DATA_ORACLE_HANDLE);
-	if (!priv_data) {
+	if (!priv_data || !ora_xaction) {
 		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
+	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
+		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		return FALSE;
+	}
+
 	/* attach the transaction to the service context */
 	result = OCIAttrSet ((dvoid *) priv_data->hservice,
 				OCI_HTYPE_SVCCTX,
@@ -915,11 +979,16 @@ gda_oracle_provider_rollback_transaction (GdaServerProvider *provider,
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	ora_xaction = g_object_get_data (G_OBJECT (xaction), OBJECT_DATA_ORACLE_HANDLE);
-	if (!priv_data) {
+	if (!priv_data || !ora_xaction) {
 		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
+	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
+		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		return FALSE;
+	}
+
 	/* attach the transaction to the service context */
 	result = OCIAttrSet ((dvoid *) priv_data->hservice,
 				OCI_HTYPE_SVCCTX,
@@ -1047,12 +1116,79 @@ get_oracle_aggregates (GdaConnection *cnc, GdaParameterList *params)
 	return GDA_DATA_MODEL (recset);
 }
 
+static GTree *
+gda_oracle_table_tree(GdaConnection *cnc)
+{
+    GdaOracleConnectionData *priv_data;
+    GTree *tree;
+    GList *reclist;
+    GdaDataModel *recset;
+    GdaRow *row;
+    GdaValue *value;
+    int i;
+    gchar *name, *owner;
+
+    reclist = process_sql_commands (NULL, cnc,
+				    "SELECT TABLE_NAME,OWNER "
+				    "FROM ALL_TABLES "
+				    "ORDER BY TABLE_NAME",
+				    NULL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
+    if (!reclist)
+	return NULL;
+    recset = reclist->data;
+    priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
+    tree = g_tree_new(strcmp);
+    for (i = 0; (row = gda_data_model_get_row(recset, i)); i++) {
+	value = gda_row_get_value(row, 0);
+	name = gda_value_stringify (value);
+	value = gda_row_get_value(row, 1);
+	owner = gda_value_stringify (value);
+	if (strcmp(owner, priv_data->schema) == 0 ||
+	    g_tree_lookup(tree, name) == NULL) {
+	    g_tree_insert(tree, name, owner);
+	}
+    }
+    priv_data->tables = tree;
+    return tree;
+}
+     
+static GTree *
+gda_oracle_view_tree(GdaConnection *cnc)
+{
+    GdaOracleConnectionData *priv_data;
+    GTree *tree;
+    GList *reclist;
+    GdaDataModel *recset;
+    GdaRow *row;
+    GdaValue *value;
+    int i;
+    gchar *name;
+
+    reclist = process_sql_commands (NULL, cnc,
+				    "SELECT VIEW_NAME "
+				    "FROM USER_VIEWS "
+				    "ORDER BY VIEW_NAME",
+				    NULL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
+    if (!reclist)
+	return NULL;
+    recset = reclist->data;
+    priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
+    tree = g_tree_new(strcmp);
+    for (i = 0; (row = gda_data_model_get_row(recset, i)); i++) {
+	value = gda_row_get_value(row, 0);
+	name = gda_value_stringify (value);
+	g_tree_insert(tree, name, priv_data->schema);
+    }
+    priv_data->views = tree;
+    return tree;
+}
+
 static GdaDataModelArray *
 gda_oracle_init_md_recset (GdaConnection *cnc)
 {
 	GdaDataModelArray *recset;
 	gint i;
-	GdaOracleColData cols[8] = {
+	GdaOracleColData cols[9] = {
 		{ N_("Field name")	, GDA_VALUE_TYPE_STRING },
 		{ N_("Data type")	, GDA_VALUE_TYPE_STRING },
 		{ N_("Size")		, GDA_VALUE_TYPE_INTEGER },
@@ -1060,7 +1196,8 @@ gda_oracle_init_md_recset (GdaConnection *cnc)
 		{ N_("Not null?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("Primary key?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("Unique index?")	, GDA_VALUE_TYPE_BOOLEAN },
-		{ N_("References")	, GDA_VALUE_TYPE_STRING }
+		{ N_("References")	, GDA_VALUE_TYPE_STRING  },
+		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  }
 	};
 
 	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (sizeof cols / sizeof cols[0]));
@@ -1210,13 +1347,27 @@ gda_oracle_fill_md_data (const gchar *tblname,
 	OCIParam *collsthd; /* handle to list of columns */
 	OCIParam *colhd;    /* column handle */
 	gint i;
+	ub1 obj_type;
 	ub2 numcols;
 	GList *list = NULL;
 	gint result;
 	GHashTable *h_table_index;
+	gchar *upc_tblname, *owner, *fq_tblname;
+	ub4 one = 1;
+	text *syn_schema, *syn_name, *syn_link;
+	ub4  syn_schema_len, syn_name_len, syn_link_len;
+	gchar *syn_points_to, *syn_ptr;
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 
+	/* get a list of table/views to resolve owner conflicts */
+	if (priv_data->tables == NULL)
+	    if (gda_oracle_table_tree(cnc) == NULL)
+		return NULL;
+	if (priv_data->views == NULL)
+	    if (gda_oracle_view_tree(cnc) == NULL)
+		return NULL;
+
 	/* Allocate the Describe handle */
 	result = OCIHandleAlloc ((dvoid *) priv_data->henv,
 				(dvoid **) &dschp,
@@ -1227,15 +1378,143 @@ gda_oracle_fill_md_data (const gchar *tblname,
 			_("Could not allocate the Oracle describe handle")))
 		return NULL;
 
+	result = OCIAttrSet ((dvoid *)dschp,
+			     (ub4) OCI_HTYPE_DESCRIBE,
+			     (dvoid *)&one,
+			     0,
+			     OCI_ATTR_DESC_PUBLIC,
+			     priv_data->herr);
+	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
+			_("Could not set describe handle attribute"))) {
+		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+		return NULL;
+	}
+
+	/* look up the table name to get the fully qualified name. */
+	upc_tblname = g_ascii_strup(tblname, -1);
+	if ((owner = g_tree_lookup(priv_data->tables, upc_tblname)) == NULL)
+	    owner = g_tree_lookup(priv_data->views, upc_tblname);
+	g_free(upc_tblname);
+	if (owner == NULL)
+	    fq_tblname = tblname;
+	else
+	    fq_tblname = g_strjoin(".", owner, tblname, NULL);
+	g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "full name is '%s'",fq_tblname);
+	
 	/* Describe the table */
 	result = OCIDescribeAny (priv_data->hservice,
 				priv_data->herr,
-				(text *) tblname,
-				strlen (tblname),
+				(text *) fq_tblname,
+				strlen (fq_tblname),
 				OCI_OTYPE_NAME,
 				0,
-				OCI_PTYPE_TABLE,
+				OCI_PTYPE_UNK,
 				(OCIDescribe *) dschp);
+	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
+			_("Could not describe the Oracle table"))) {
+		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+		if (fq_tblname != tblname)
+		    g_free(fq_tblname);
+		return NULL;
+	}
+
+	/* Get the parameter handle */
+	result = OCIAttrGet ((dvoid *) dschp,
+				(ub4) OCI_HTYPE_DESCRIBE, 
+				(dvoid **) &parmh,
+				(ub4 *) 0,
+				(ub4) OCI_ATTR_PARAM,
+				(OCIError *) priv_data->herr);
+	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
+			_("Could not get the Oracle parameter handle"))) {
+		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+		if (fq_tblname != tblname)
+		    g_free(fq_tblname);
+		return NULL;
+	}
+
+	/* Find what type of thing we actually described */
+	result = OCIAttrGet ((dvoid *) parmh,
+				(ub4) OCI_DTYPE_PARAM,
+				(dvoid *) &obj_type,
+				(ub4 *) 0,
+				(ub4) OCI_ATTR_PTYPE,
+				(OCIError *) priv_data->herr);
+	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
+			_("Could not get described object type"))) {
+		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+		if (fq_tblname != tblname)
+		    g_free(fq_tblname);
+		return NULL;
+	}
+	if (obj_type == OCI_PTYPE_SYN) {
+	    /* get the name the synonym points to */
+	    result = OCIAttrGet ((dvoid *) parmh,
+				 (ub4) OCI_DTYPE_PARAM,
+				 (dvoid *) &syn_schema,
+				 (ub4 *) &syn_schema_len,
+				 (ub4) OCI_ATTR_SCHEMA_NAME,
+				 (OCIError *) priv_data->herr);
+	    if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
+			_("Could not get synonym referred-to schema"))) {
+		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+		if (fq_tblname != tblname)
+		    g_free(fq_tblname);
+		return NULL;
+	    }
+	    result = OCIAttrGet ((dvoid *) parmh,
+				 (ub4) OCI_DTYPE_PARAM,
+				 (dvoid *) &syn_name,
+				 (ub4 *) &syn_name_len,
+				 (ub4) OCI_ATTR_NAME,
+				 (OCIError *) priv_data->herr);
+	    if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
+			_("Could not get synonym referred-to name"))) {
+		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+		if (fq_tblname != tblname)
+		    g_free(fq_tblname);
+		return NULL;
+	    }
+	    result = OCIAttrGet ((dvoid *) parmh,
+				 (ub4) OCI_DTYPE_PARAM,
+				 (dvoid *) &syn_link,
+				 (ub4 *) &syn_link_len,
+				 (ub4) OCI_ATTR_LINK,
+				 (OCIError *) priv_data->herr);
+	    if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
+			_("Could not get synonym referred-to dblink"))) {
+		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+		if (fq_tblname != tblname)
+		    g_free(fq_tblname);
+		return NULL;
+	    }
+	    /* combine the synonym attributes into a single name */
+	    if (fq_tblname != tblname)
+		g_free(fq_tblname);
+	    fq_tblname = g_malloc(syn_schema_len+syn_name_len+syn_link_len+4);
+	    strncpy(fq_tblname, syn_schema, syn_schema_len);
+	    syn_ptr = fq_tblname + syn_schema_len;
+	    *syn_ptr++ = '.';
+	    strncpy(syn_ptr, syn_name, syn_name_len);
+	    syn_ptr += syn_name_len;
+	    if (syn_link != NULL)
+	    {
+		*syn_ptr++ = '@';
+		strncpy(syn_ptr, syn_link, syn_link_len);
+		syn_ptr += syn_link_len;
+	    }
+	    *syn_ptr = '\0';
+
+	    /* re-issue the OCIDescribeAny for the referred to object. */
+	    result = OCIDescribeAny (priv_data->hservice,
+				     priv_data->herr,
+				     (text *) syn_points_to,
+				     syn_ptr - syn_points_to,
+				     OCI_OTYPE_NAME,
+				     0,
+				     OCI_PTYPE_UNK,
+				(OCIDescribe *) dschp);
+	    g_free(syn_points_to);
 	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
 			_("Could not describe the Oracle table"))) {
 		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
@@ -1252,8 +1531,10 @@ gda_oracle_fill_md_data (const gchar *tblname,
 	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
 			_("Could not get the Oracle parameter handle"))) {
 		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+		g_free(fq_tblname);
 		return NULL;
 	}
+	}
 
 	/* Get the number of columns */
 	result = OCIAttrGet ((dvoid *) parmh,
@@ -1265,6 +1546,8 @@ gda_oracle_fill_md_data (const gchar *tblname,
 	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
 			_("Could not get the number of columns in the table"))) {
 		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+		if (fq_tblname != tblname)
+		    g_free(fq_tblname);
 		return NULL;
 	}
 
@@ -1277,10 +1560,12 @@ gda_oracle_fill_md_data (const gchar *tblname,
 	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
 			_("Could not get the column list"))) {
 		OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+		if (fq_tblname != tblname)
+		    g_free(fq_tblname);
 		return NULL;
 	}
 
-	h_table_index = get_oracle_index_data (cnc, tblname);
+	h_table_index = get_oracle_index_data (cnc, fq_tblname);
 
 	for (i = 1; i <= numcols; i += 1) {
 		text *strp;
@@ -1306,6 +1591,8 @@ gda_oracle_fill_md_data (const gchar *tblname,
 		if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
 				_("Could not get the Oracle column handle"))) {
 			OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+			if (fq_tblname != tblname)
+			    g_free(fq_tblname);
 			return NULL;
 		}
 		
@@ -1320,6 +1607,8 @@ gda_oracle_fill_md_data (const gchar *tblname,
 				_("Could not get the Oracle field name"))) {
 			OCIDescriptorFree ((dvoid *) colhd, OCI_DTYPE_PARAM);
 			OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+			if (fq_tblname != tblname)
+			    g_free(fq_tblname);
 			return NULL;
 		}
 
@@ -1330,7 +1619,6 @@ gda_oracle_fill_md_data (const gchar *tblname,
 		rowlist = g_list_append (rowlist, value);
 
 		/* Data type */
-		typename = g_malloc0 (31);
 		result = OCIAttrGet ((dvoid *)colhd,
 					(ub4) OCI_DTYPE_PARAM,
 					(dvoid *) &type,
@@ -1341,6 +1629,8 @@ gda_oracle_fill_md_data (const gchar *tblname,
 				_("Could not get the Oracle field data type"))) {
 			OCIDescriptorFree ((dvoid *) colhd, OCI_DTYPE_PARAM);
 			OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+			if (fq_tblname != tblname)
+			    g_free(fq_tblname);
 			return NULL;
 		}
 
@@ -1359,6 +1649,8 @@ gda_oracle_fill_md_data (const gchar *tblname,
 				_("Could not get the Oracle field defined size"))) {
 			OCIDescriptorFree ((dvoid *) colhd, OCI_DTYPE_PARAM);
 			OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+			if (fq_tblname != tblname)
+			    g_free(fq_tblname);
 			return NULL;
 		}
 
@@ -1376,6 +1668,8 @@ gda_oracle_fill_md_data (const gchar *tblname,
 				_("Could not get the Oracle field scale"))) {
 			OCIDescriptorFree ((dvoid *) colhd, OCI_DTYPE_PARAM);
 			OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+			if (fq_tblname != tblname)
+			    g_free(fq_tblname);
 			return NULL;
 		}
 
@@ -1393,6 +1687,8 @@ gda_oracle_fill_md_data (const gchar *tblname,
 				_("Could not get the Oracle field nullable attribute"))) {
 			OCIDescriptorFree ((dvoid *) colhd, OCI_DTYPE_PARAM);
 			OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+			if (fq_tblname != tblname)
+			    g_free(fq_tblname);
 			return NULL;
 		}
 
@@ -1427,6 +1723,10 @@ gda_oracle_fill_md_data (const gchar *tblname,
 		value = gda_value_new_string (index_data->references);
 		rowlist = g_list_append (rowlist, value);
 
+		/* default */
+		value = gda_value_new_string ("none");
+		rowlist = g_list_append (rowlist, value);
+		
 		g_free (index_data);
 
 		list = g_list_append (list, rowlist);
@@ -1436,6 +1736,8 @@ gda_oracle_fill_md_data (const gchar *tblname,
 	}
 
 	OCIHandleFree ((dvoid *) dschp, OCI_HTYPE_DESCRIBE);
+	if (fq_tblname != tblname)
+	    g_free(fq_tblname);
 	return list;
 }
 
@@ -1483,6 +1785,150 @@ get_oracle_databases (GdaConnection *cnc, GdaParameterList *params)
 
 	return recset;
 }
+
+static gboolean
+get_tables_foreach(gpointer key, gpointer value, gpointer data)
+{
+    GList *cols = NULL;
+    
+    cols = g_list_append(cols, gda_value_new_string((const gchar *)key));
+    cols = g_list_append(cols, gda_value_new_string((const gchar *)value));
+    cols = g_list_append(cols, gda_value_new_string(""));
+    cols = g_list_append(cols, gda_value_new_string(""));
+    gda_data_model_append_row((GdaDataModel *)data, cols);
+    return FALSE;
+}
+
+static GdaDataModel *
+get_oracle_tables (GdaConnection *cnc, GdaParameterList *params)
+{
+	GdaDataModel *recset;
+    GdaParameter *par = NULL;
+    const gchar  *namespace, *upc_namespace;
+    GList *reclist;
+    gchar *sql;
+    GdaOracleConnectionData *priv_data;
+    gint i;
+
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+
+    if (params)
+	par = gda_parameter_list_find (params, "namespace");
+    if (par)
+    {
+	g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "parameterised get_tables");
+	namespace = gda_value_get_string ((GdaValue *) gda_parameter_get_value (par));
+	upc_namespace = g_ascii_strup(namespace, -1);
+
+	sql = g_strdup_printf ("SELECT TABLE_NAME AS \"%s\","
+			       "OWNER AS \"%s\","
+			       "NULL AS \"%s\","
+			       "NULL AS \"SQL\" "
+			       "FROM ALL_TABLES "
+			       "WHERE OWNER=\"%s\" "
+			       "ORDER BY TABLE_NAME",
+			       _("Name"), _("Owner"), _("Comments"),
+			       upc_namespace);
+
+	reclist = process_sql_commands (NULL, cnc, sql, NULL, 
+					GDA_COMMAND_OPTION_STOP_ON_ERRORS);
+	g_free(upc_namespace);
+	g_free (sql);
+
+	if (!reclist)
+		return NULL;
+
+	recset = GDA_DATA_MODEL (reclist->data);
+	g_list_free (reclist);
+    }
+    else
+    {
+	GdaOracleColData cols[4] = {
+		{ N_("Name"),     GDA_VALUE_TYPE_STRING },
+		{ N_("Owner"),    GDA_VALUE_TYPE_STRING },
+		{ N_("Comments"), GDA_VALUE_TYPE_STRING },
+		{ N_("SQL"),      GDA_VALUE_TYPE_STRING }
+	};
+	g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "default get_tables");
+	priv_data = g_object_get_data(G_OBJECT(cnc),OBJECT_DATA_ORACLE_HANDLE);
+	if (priv_data->tables == NULL)
+	    if (gda_oracle_table_tree(cnc) == NULL)
+		return NULL;
+	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (sizeof cols / sizeof cols[0]));
+	for (i = 0; i < sizeof cols / sizeof cols[0]; i += 1)
+	    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), i, _(cols[i].col_name));
+
+	g_tree_foreach(priv_data->tables, get_tables_foreach, recset);
+    }
+    g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "get_tables returning %p", recset);
+    return recset;
+}
+
+static GdaDataModel *
+get_oracle_views (GdaConnection *cnc, GdaParameterList *params)
+{
+    GdaDataModel *recset;
+    GdaParameter *par = NULL;
+    const gchar  *namespace, *upc_namespace;
+    GList *reclist;
+    gchar *sql;
+    GdaOracleConnectionData *priv_data;
+    gint i;
+    
+    g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	
+    if (params)
+	par = gda_parameter_list_find (params, "namespace");
+    if (par)
+    {
+	g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "parameterised get_views");
+	namespace = gda_value_get_string ((GdaValue *) gda_parameter_get_value (par));
+	upc_namespace = g_ascii_strup(namespace, -1);
+
+	sql = g_strdup_printf ("SELECT VIEW_NAME AS \"%s\","
+			       "OWNER AS \"%s\","
+			       "NULL AS \"%s\","
+			       "NULL AS \"SQL\" "
+			       "FROM ALL_VIEWS "
+			       "WHERE OWNER=\"%s\" "
+			       "ORDER BY VIEW_NAME",
+			       _("Name"), _("Owner"), _("Comments"),
+			       upc_namespace);
+
+	reclist = process_sql_commands (NULL, cnc, sql, NULL, 
+					GDA_COMMAND_OPTION_STOP_ON_ERRORS);
+	g_free(upc_namespace);
+	g_free (sql);
+
+	if (!reclist)
+		return NULL;
+	
+	recset = GDA_DATA_MODEL (reclist->data);
+	g_list_free (reclist);
+    }
+    else
+    {
+	GdaOracleColData cols[4] = {
+		{ N_("Name"),     GDA_VALUE_TYPE_STRING },
+		{ N_("Owner"),    GDA_VALUE_TYPE_STRING },
+		{ N_("Comments"), GDA_VALUE_TYPE_STRING },
+		{ N_("SQL"),      GDA_VALUE_TYPE_STRING }
+	};
+	g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "default get_views");
+	priv_data = g_object_get_data(G_OBJECT(cnc),OBJECT_DATA_ORACLE_HANDLE);
+	if (priv_data->views == NULL)
+	    if (gda_oracle_view_tree(cnc) == NULL)
+		return NULL;
+	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (sizeof cols / sizeof cols[0]));
+	for (i = 0; i < sizeof cols / sizeof cols[0]; i += 1)
+	    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), i, _(cols[i].col_name));
+	
+	g_tree_foreach(priv_data->views, get_tables_foreach, recset);
+    }
+    g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "get_views returning %p", recset);
+	return recset;
+}
+
 /*
  * get_oracle_objects
  *
@@ -1497,50 +1943,53 @@ get_oracle_databases (GdaConnection *cnc, GdaParameterList *params)
  * see any use for any user passed parameters at this point.
  */
 static GdaDataModel *
-get_oracle_objects (GdaConnection *cnc, GdaParameterList *params, GdaConnectionSchema schema)
+get_oracle_objects (GdaConnection *cnc, GdaParameterList *params, GdaConnectionSchema schema, gint nargs)
 {
 	GList *reclist;
 	GdaDataModel *recset;
+	gint cnt;
 	GdaParameterList *query_params = gda_parameter_list_new ();
-        gchar *sql = g_strdup ("SELECT OBJECT_NAME FROM USER_OBJECTS WHERE OBJECT_TYPE=:OBJ_TYPE");
+        GString *sql;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
+	sql = g_string_new ("SELECT OBJECT_NAME");
+	for (cnt = 1; cnt < nargs; cnt++)
+		sql = g_string_append (sql, ", NULL");
+
+	sql = g_string_append (sql, " FROM USER_OBJECTS WHERE OBJECT_TYPE=:OBJ_TYPE");
+
 	/* add the object type parameter */
 	switch (schema) {
 	case GDA_CONNECTION_SCHEMA_INDEXES:
 		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "INDEX"));
+						  gda_parameter_new_string (":OBJ_TYPE", "INDEX"));
 		break;
 	case GDA_CONNECTION_SCHEMA_PROCEDURES:
 		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "PROCEDURE"));
+						  gda_parameter_new_string (":OBJ_TYPE", "PROCEDURE"));
 		break;
 	case GDA_CONNECTION_SCHEMA_SEQUENCES:
 		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "SEQUENCE"));
-		break;
-	case GDA_CONNECTION_SCHEMA_TABLES:
-		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "TABLE"));
+						  gda_parameter_new_string (":OBJ_TYPE", "SEQUENCE"));
 		break;
 	case GDA_CONNECTION_SCHEMA_TRIGGERS:
 		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "TRIGGER"));
+						  gda_parameter_new_string (":OBJ_TYPE", "TRIGGER"));
 		break;
 	case GDA_CONNECTION_SCHEMA_VIEWS:
 		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "VIEW"));
+						  gda_parameter_new_string (":OBJ_TYPE", "VIEW"));
 		break;
 	default:
 		return NULL;
 	}
 
-	reclist = process_sql_commands (NULL, cnc, sql, query_params,
-			GDA_COMMAND_OPTION_STOP_ON_ERRORS);
+	reclist = process_sql_commands (NULL, cnc, sql->str, query_params,
+					GDA_COMMAND_OPTION_STOP_ON_ERRORS);
 
 	gda_parameter_list_free (query_params);
-	g_free (sql);
+	g_string_free (sql, TRUE);
 
 	if (!reclist)
 		return NULL;
@@ -1647,27 +2096,26 @@ gda_oracle_provider_get_schema (GdaServerProvider *provider,
 		recset = get_oracle_databases (cnc, params);
 		break;
 	case GDA_CONNECTION_SCHEMA_INDEXES :
-		recset = get_oracle_objects (cnc, params, schema);
+		recset = get_oracle_objects (cnc, params, schema, 1);
 		gda_data_model_set_column_title (recset, 0, _("Indexes"));
 		break;
 	case GDA_CONNECTION_SCHEMA_PROCEDURES :
-		recset = get_oracle_objects (cnc, params, schema);
+		recset = get_oracle_objects (cnc, params, schema, 8);
 		gda_data_model_set_column_title (recset, 0, _("Procedures"));
 		break;
 	case GDA_CONNECTION_SCHEMA_SEQUENCES :
-		recset = get_oracle_objects (cnc, params, schema);
+		recset = get_oracle_objects (cnc, params, schema, 4);
 		gda_data_model_set_column_title (recset, 0, _("Sequences"));
 		break;
 	case GDA_CONNECTION_SCHEMA_TABLES :
-		recset = get_oracle_objects (cnc, params, schema);
-		gda_data_model_set_column_title (recset, 0, _("Tables"));
+		recset = get_oracle_tables (cnc, params);
 		break;
 	case GDA_CONNECTION_SCHEMA_TRIGGERS :
-		recset = get_oracle_objects (cnc, params, schema);
+		recset = get_oracle_objects (cnc, params, schema, 1);
 		gda_data_model_set_column_title (recset, 0, _("Triggers"));
 		break;
 	case GDA_CONNECTION_SCHEMA_VIEWS :
-		recset = get_oracle_objects (cnc, params, schema);
+		recset = get_oracle_views(cnc, params);
 		gda_data_model_set_column_title (recset, 0, _("Views"));
 		break;
 	default :
